@@ -26,12 +26,6 @@
 #include <string.h>
 #include <assert.h>
 #include <errno.h>
-#include <time.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <signal.h>
-#include <fcntl.h>
-#include "unistd.h"
 
 #include "gpgme.h"
 #include "util.h"
@@ -282,6 +276,33 @@ free_fd_data_map (struct fd_data_map_s *fd_data_map)
 }
 
 
+static gpgme_error_t
+gpg_cancel (void *engine)
+{
+  engine_gpg_t gpg = engine;
+
+  if (!gpg)
+    return gpg_error (GPG_ERR_INV_VALUE);
+
+  if (gpg->status.fd[0] != -1)
+    _gpgme_io_close (gpg->status.fd[0]);
+  if (gpg->status.fd[1] != -1)
+    _gpgme_io_close (gpg->status.fd[1]);
+  if (gpg->colon.fd[0] != -1)
+    _gpgme_io_close (gpg->colon.fd[0]);
+  if (gpg->colon.fd[1] != -1)
+    _gpgme_io_close (gpg->colon.fd[1]);
+  if (gpg->fd_data_map)
+    {
+      free_fd_data_map (gpg->fd_data_map);
+      gpg->fd_data_map = NULL;
+    }
+  if (gpg->cmd.fd != -1)
+    _gpgme_io_close (gpg->cmd.fd);
+
+  return 0;
+}
+
 static void
 gpg_release (void *engine)
 {
@@ -290,6 +311,8 @@ gpg_release (void *engine)
   if (!gpg)
     return;
 
+  gpg_cancel (engine);
+
   while (gpg->arglist)
     {
       struct arg_and_data_s *next = gpg->arglist->next;
@@ -308,24 +331,12 @@ gpg_release (void *engine)
   if (gpg->cmd.keyword)
     free (gpg->cmd.keyword);
 
-  if (gpg->status.fd[0] != -1)
-    _gpgme_io_close (gpg->status.fd[0]);
-  if (gpg->status.fd[1] != -1)
-    _gpgme_io_close (gpg->status.fd[1]);
-  if (gpg->colon.fd[0] != -1)
-    _gpgme_io_close (gpg->colon.fd[0]);
-  if (gpg->colon.fd[1] != -1)
-    _gpgme_io_close (gpg->colon.fd[1]);
-  if (gpg->fd_data_map)
-    free_fd_data_map (gpg->fd_data_map);
-  if (gpg->cmd.fd != -1)
-    _gpgme_io_close (gpg->cmd.fd);
   free (gpg);
 }
 
 
 static gpgme_error_t
-gpg_new (void **engine)
+gpg_new (void **engine, const char *lc_ctype, const char *lc_messages)
 {
   engine_gpg_t gpg;
   gpgme_error_t rc = 0;
@@ -503,6 +514,7 @@ gpg_set_command_handler (void *engine, engine_command_handler_t fnc,
 static gpgme_error_t
 build_argv (engine_gpg_t gpg)
 {
+  gpgme_error_t err;
   struct arg_and_data_s *a;
   struct fd_data_map_s *fd_data_map;
   size_t datac=0, argc=0;  
@@ -514,9 +526,13 @@ build_argv (engine_gpg_t gpg)
   /* We don't want to use the agent with a malformed environment
      variable.  This is only a very basic test but sufficient to make
      our life in the regression tests easier. */
-  p = getenv ("GPG_AGENT_INFO");
+  err = _gpgme_getenv ("GPG_AGENT_INFO", &p);
+  if (err)
+    return err;
   use_agent = (p && strchr (p, ':'));
-       
+  if (p)
+    free (p);
+
   if (gpg->argv)
     {
       free_argv (gpg->argv);
@@ -550,7 +566,7 @@ build_argv (engine_gpg_t gpg)
     argc++;
   if (!gpg->cmd.used)
     argc++;	/* --batch */
-  argc += 2; /* --comment */
+  argc += 1;	/* --no-sk-comment */
 
   argv = calloc (argc + 1, sizeof *argv);
   if (!argv)
@@ -609,16 +625,7 @@ build_argv (engine_gpg_t gpg)
         }
       argc++;
     }
-  argv[argc] = strdup ("--comment");
-  if (!argv[argc])
-    {
-      int saved_errno = errno;
-      free (fd_data_map);
-      free_argv (argv);
-      return gpg_error_from_errno (saved_errno);
-    }
-  argc++;
-  argv[argc] = strdup ("");
+  argv[argc] = strdup ("--no-sk-comment");
   if (!argv[argc])
     {
       int saved_errno = errno;
@@ -834,29 +841,6 @@ read_status (engine_gpg_t gpg)
 			  if (nread > 1)
 			    DEBUG0 ("ERROR, unexpected data in read_status");
 
-			  /* Before we can actually add the command
-			     fd, we might have to flush the linked
-			     output data pipe.  */
-			  if (gpg->cmd.linked_idx != -1
-			      && gpg->fd_data_map[gpg->cmd.linked_idx].fd != -1)
-			    {
-			      struct io_select_fd_s fds;
-			      fds.fd = gpg->fd_data_map[gpg->cmd.linked_idx].fd;
-			      fds.for_read = 1;
-			      fds.for_write = 0;
-			      fds.frozen = 0;
-			      fds.opaque = NULL;
-			      do
-				{
-				  fds.signaled = 0;
-				  _gpgme_io_select (&fds, 1, 1);
-				  if (fds.signaled)
-				    _gpgme_data_inbound_handler
-				      (gpg->cmd.linked_data, fds.fd);
-				}
-			      while (fds.signaled);
-			    }
-
 			  add_io_cb (gpg, gpg->cmd.fd, 0,
 				     command_handler, gpg,
 				     &gpg->fd_data_map[gpg->cmd.idx].tag);
@@ -876,6 +860,31 @@ read_status (engine_gpg_t gpg)
 			{
 			  if (gpg->cmd.used)
 			    {
+			      /* Before we can actually add the
+				 command fd, we might have to flush
+				 the linked output data pipe.  */
+			      if (gpg->cmd.linked_idx != -1
+				  && gpg->fd_data_map[gpg->cmd.linked_idx].fd
+				  != -1)
+				{
+				  struct io_select_fd_s fds;
+				  fds.fd =
+				    gpg->fd_data_map[gpg->cmd.linked_idx].fd;
+				  fds.for_read = 1;
+				  fds.for_write = 0;
+				  fds.frozen = 0;
+				  fds.opaque = NULL;
+				  do
+				    {
+				      fds.signaled = 0;
+				      _gpgme_io_select (&fds, 1, 1);
+				      if (fds.signaled)
+					_gpgme_data_inbound_handler
+					  (gpg->cmd.linked_data, fds.fd);
+				    }
+				  while (fds.signaled);
+				}
+
 			      /* XXX We must check if there are any
 				 more fds active after removing this
 				 one.  */
@@ -1097,7 +1106,7 @@ start (engine_gpg_t gpg)
   saved_errno = errno;
   free (fd_child_list);
   if (status == -1)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_errno (saved_errno);
 
   /*_gpgme_register_term_handler ( closure, closure_value, pid );*/
 
@@ -1219,7 +1228,7 @@ append_args_from_signers (engine_gpg_t gpg, gpgme_ctx_t ctx /* FIXME */)
 
 
 static gpgme_error_t
-gpg_edit (void *engine, gpgme_key_t key, gpgme_data_t out,
+gpg_edit (void *engine, int type, gpgme_key_t key, gpgme_data_t out,
 	  gpgme_ctx_t ctx /* FIXME */)
 {
   engine_gpg_t gpg = engine;
@@ -1229,7 +1238,7 @@ gpg_edit (void *engine, gpgme_key_t key, gpgme_data_t out,
   if (!err)
     err = append_args_from_signers (gpg, ctx);
   if (!err)
-  err = add_arg (gpg, "--edit-key");
+  err = add_arg (gpg, type == 0 ? "--edit-key" : "--card-edit");
   if (!err)
     err = add_data (gpg, out, 1, 1);
   if (!err)
@@ -1674,5 +1683,6 @@ struct engine_ops _gpgme_engine_ops_gpg =
     gpg_trustlist,
     gpg_verify,
     gpg_set_io_cbs,
-    gpg_io_event
+    gpg_io_event,
+    gpg_cancel
   };
