@@ -18,6 +18,13 @@
    along with GPGME; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+
+/* We need to include config.h so that we know whether we are building
+   with large file system (LFS) support. */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -49,6 +56,8 @@ main (int argc, char *argv[])
   gpgme_data_t in, out;
   gpgme_decrypt_result_t result;
 
+  init_gpgme (GPGME_PROTOCOL_CMS);
+
   err = gpgme_new (&ctx);
   fail_if_err (err);
   gpgme_set_protocol (ctx, GPGME_PROTOCOL_CMS);
@@ -75,5 +84,3 @@ main (int argc, char *argv[])
   gpgme_release (ctx);
   return 0;
 }
-
-
