@@ -18,6 +18,12 @@
    along with GPGME; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+/* We need to include config.h so that we know whether we are building
+   with large file system (LFS) support. */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -25,44 +31,12 @@
 
 #include <gpgme.h>
 
-
-#define fail_if_err(err)					\
-  do								\
-    {								\
-      if (err)							\
-        {							\
-          fprintf (stderr, "%s:%d: gpgme_error_t %s\n",		\
-                   __FILE__, __LINE__, gpgme_strerror (err));   \
-          exit (1);						\
-        }							\
-    }								\
-  while (0)
-
-
-static char *
-make_filename (const char *fname)
-{
-  const char *srcdir = getenv ("srcdir");
-  char *buf;
-
-  if (!srcdir)
-    srcdir = ".";
-  buf = malloc (strlen(srcdir) + strlen(fname) + 2);
-  if (!buf)
-    {
-      fprintf (stderr, "%s:%d: could not allocate string: %s\n",
-	       __FILE__, __LINE__, strerror (errno));
-      exit (1);
-    }
-  strcpy (buf, srcdir);
-  strcat (buf, "/");
-  strcat (buf, fname);
-  return buf;
-}
+#include "t-support.h"
 
 
 void
-check_result (gpgme_import_result_t result, char *fpr, int total)
+check_result (gpgme_import_result_t result, char *fpr, int total,
+	      int total_stat)
 {
   if (result->considered != total)
     {
@@ -143,11 +117,20 @@ check_result (gpgme_import_result_t result, char *fpr, int total)
 	       result->not_imported);
       exit (1);
     }
-  if (result->imports)
+  
+  {
+    int n;
+    gpgme_import_status_t r;
+
+    for (n=0, r=result->imports; r; r=r->next)
+      n++;
+      
+    if (n != total_stat)
     {
       fprintf (stderr, "Unexpected number of status reports\n");
       exit (1);
     }
+  }
 }
 
 
@@ -161,8 +144,7 @@ main (int argc, char **argv)
   const char *cert_1 = make_filename ("cert_dfn_pca01.der");
   const char *cert_2 = make_filename ("cert_dfn_pca15.der");
 
-  err = gpgme_engine_check_version (GPGME_PROTOCOL_CMS);
-  fail_if_err (err);
+  init_gpgme (GPGME_PROTOCOL_CMS);
 
   err = gpgme_new (&ctx);
   fail_if_err (err);
@@ -175,7 +157,7 @@ main (int argc, char **argv)
   err = gpgme_op_import (ctx, in);
   fail_if_err (err);
   result = gpgme_op_import_result (ctx);
-  check_result (result, "DFA56FB5FC41E3A8921F77AD1622EEFD9152A5AD", 1);
+  check_result (result, "DFA56FB5FC41E3A8921F77AD1622EEFD9152A5AD", 1, 1);
   gpgme_data_release (in);
 
   err = gpgme_data_new_from_file (&in, cert_2, 1);
@@ -184,7 +166,7 @@ main (int argc, char **argv)
   err = gpgme_op_import (ctx, in);
   fail_if_err (err);
   result = gpgme_op_import_result (ctx);
-  check_result (result, "2C8F3C356AB761CB3674835B792CDA52937F9285", 2);
+  check_result (result, "2C8F3C356AB761CB3674835B792CDA52937F9285", 1, 2);
   gpgme_data_release (in);
 
   gpgme_release (ctx);
